/**
 ****************************************************************************************
 *
 * @file app_blps.c
 *
 * @brief Blood Pressure Module entry point
 *
 * Copyright (C) Beken 2009-2016
 *
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup APP
 * @{
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration

#if (BLE_BP_SENSOR)

/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include <string.h>
#include "app_blps.h"                // 
#include "app.h"                     // Application Definitions
#include "app_task.h"                // application task definitions
#include "blps_task.h"               // Blood Pressure functions
#include "co_bt.h"
#include "prf_types.h"               // Profile common types definition
#include "arch.h"                    // Platform Definitions
#include "prf.h"
#include "blps.h"

#include "uart.h"
#include "ke_timer.h"
#include "user_config.h"
/*
 * DEFINES
 ****************************************************************************************
 */

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Blood Pressure Application Module Environment Structure
struct app_blps_env_tag app_blps_env;

uint8_t blps_reprot_count=0;

/*
 * GLOBAL FUNCTION DEFINITIONS
 ****************************************************************************************
 */

void app_blps_init(void)
{
    // Reset the environment
    memset(&app_blps_env, 0, sizeof(struct app_blps_env_tag));

    // Initial ....
    DBG_PRINTF("[BLPS] %s \r\n", __func__);
}




/* Blood Pressure Feature Bit 
0 Body Movement Detection Support bit 
1 Cuff Fit Detection Support bit 
2 Irregular Pulse Detection Support bit
3 Pulse Rate Range Detection Support bit 
4 Measurement Position Detection Support bit
5 Multiple Bond Support bit 
6 E2E-CRC Support bit
7 User Data Service Support bit
8 User Facing Time Support bit 
*/
void app_blps_add_bp(void)
{
    struct blps_db_cfg* db_cfg;
    // Allocate 
    struct gapm_profile_task_add_cmd *req = KE_MSG_ALLOC_DYN(GAPM_PROFILE_TASK_ADD_CMD,
                                                  TASK_GAPM, TASK_APP,
                                                  gapm_profile_task_add_cmd, sizeof(struct blps_db_cfg));
    DBG_PRINTF("[BLPS] %s \r\n", __func__);
    // Fill message
    req->operation   = GAPM_PROFILE_TASK_ADD;
    req->sec_lvl     = 0;
    req->prf_task_id = TASK_ID_BLPS;
    req->app_task    = TASK_APP;
    req->start_hdl   = 0;

    // Set parameters
    db_cfg = (struct blps_db_cfg* ) req->param;
 
    db_cfg->features = 2;    
    db_cfg->prfl_cfg = 0;
    // Send the message
    ke_msg_send(req);
}


static int blps_enable_rsp_handler(ke_msg_id_t const msgid,
                                    struct blps_enable_rsp const *param,
                                    ke_task_id_t const dest_id,
                                    ke_task_id_t const src_id)
{
    DBG_PRINTF("%s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

/**
 ****************************************************************************************
 * @brief
 *
 * @param[in] msgid     Id of the message received.
 * @param[in] param     Pointer to the parameters of the message.
 * @param[in] dest_id   ID of the receiving task instance (TASK_GAP).
 * @param[in] src_id    ID of the sending task instance.
 *
 * @return If the message was consumed or not.
 ****************************************************************************************
 */
static int app_blps_msg_dflt_handler(ke_msg_id_t const msgid,
                                     void const *param,
                                     ke_task_id_t const dest_id,
                                     ke_task_id_t const src_id)
{
    // Drop the message
    DBG_PRINTF("%s(msgId:0x%X) \r\n", __func__, msgid);

    return (KE_MSG_CONSUMED);
}


////Add for Notify blps data
void app_blps_send_data(void)
{
	struct blps_meas_send_req *req = KE_MSG_ALLOC(BLPS_MEAS_SEND_REQ,
                                                  prf_get_task_from_id(TASK_ID_BLPS),
                                                  TASK_APP,
                                                  blps_meas_send_req);
    DBG_PRINTF("%s (%x)\r\n", __func__,blps_reprot_count);

#if BQB_TEST_ENABLE
    //if(blps_reprot_count)
    //{
    //    ke_timer_set(APP_BLPS_MEAS_TIMER,TASK_APP,500);
    //    blps_reprot_count--;
    //}

    req->conidx = 0;//conidx;
    req->flag_interm_cp = 0;      /// PTS test = 0

	req->meas_val.flags = BPS_FLAG_MMHG;
    req->meas_val.user_id = 0;
    req->meas_val.systolic = 100;
    req->meas_val.diastolic = 100;
    req->meas_val.mean_arterial_pressure = 100;
    req->meas_val.pulse_rate = 72;
    req->meas_val.meas_status = BPS_STATE_NO_BODY_MVMT_DETECTED;
    req->meas_val.time_stamp.year = 2021;
    req->meas_val.time_stamp.month = 1;
    req->meas_val.time_stamp.day = 1;
    req->meas_val.time_stamp.hour = 12;
    req->meas_val.time_stamp.min = 0;
    req->meas_val.time_stamp.sec = 0;
#endif

	// Send the message
    ke_msg_send(req);
}



static int blps_meas_send_rsp_handler(ke_msg_id_t const msgid,
                                    struct blps_meas_send_rsp const *param,
                                    ke_task_id_t const dest_id,
                                    ke_task_id_t const src_id)
{
    DBG_PRINTF("%s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}
////Add for Indication and Notifying config action
static int app_blps_cfg_indntf_handler(ke_msg_id_t const msgid,
                                     struct blps_cfg_indntf_ind *param,
                                     ke_task_id_t const dest_id,
                                     ke_task_id_t const src_id)
{

    DBG_PRINTF("%s(conidx:%d, Code:%d, cfg:%d ) \r\n", __func__, param->conidx, param->char_code, param->cfg_val);

	if(param->char_code == BPS_BP_MEAS_CHAR)
	{	////Indication
        if( param->cfg_val & BLPS_BP_MEAS_IND_CFG)
		{
        #if BQB_TEST_ENABLE
            //blps_reprot_count = 3 ;
            ke_timer_set(APP_BLPS_MEAS_TIMER,TASK_APP,500);
            ////app_blps_send_data();
        #endif
		}
		else
        {
            //ke_timer_clear(APP_BLPS_MEAS_TIMER,TASK_APP);
		}
	}
	else if(param->char_code == BPS_INTM_CUFF_MEAS_CHAR)
	{	////Notify
		}
    else if( param->char_code == BPS_BP_FEATURE_CHAR)
    {   ///
	}
    return (KE_MSG_CONSUMED);
	}

#if(BQB_TEST_ENABLE == 1)
  #include "prf_utils.h"
  void app_blps_set_feature_val(uint16_t val)
  {
	struct blps_env_tag *blps_env = PRF_ENV_GET(BLPS, blps);
    blps_env->features = val;
}
#endif

/*
 * LOCAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Default State handlers definition
const struct ke_msg_handler app_blps_msg_handler_list[] =
{
    // Note: first message is latest message checked by kernel so default is put on top.
    {KE_MSG_DEFAULT_HANDLER,        (ke_msg_func_t)app_blps_msg_dflt_handler},
	{BLPS_ENABLE_RSP,       		(ke_msg_func_t)blps_enable_rsp_handler},
	{BLPS_MEAS_SEND_RSP,       		(ke_msg_func_t)blps_meas_send_rsp_handler},
    {BLPS_CFG_INDNTF_IND,			(ke_msg_func_t)app_blps_cfg_indntf_handler}
};

const struct ke_state_handler app_blps_table_handler =
    {&app_blps_msg_handler_list[0], (sizeof(app_blps_msg_handler_list)/sizeof(struct ke_msg_handler))};

#endif //#if (BLE_BP_SENSOR)
    
/// @} APP
