/**
 ****************************************************************************************
 *
 * @file app_blps.h
 *
 * @brief Blood Pressure Module entry point
 *
 * Copyright (C) 
 *
 *
 ****************************************************************************************
 */

#ifndef APP_BLPS_H_
#define APP_BLPS_H_

/**
 ****************************************************************************************
 * @addtogroup APP
 * @ingroup RICOW
 *
 * @brief Blood Pressure Application Module entry point
 *
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration


#include <stdint.h>          // Standard Integer Definition
#include "ke_task.h"         // Kernel Task Definition

/*
 * STRUCTURES DEFINITION
 ****************************************************************************************
 */

/// Blood Pressure Application Module Environment Structure
struct app_blps_env_tag
{
    /// Connection handle
    uint8_t conidx;
    /// Current Battery Level
    uint8_t htpt_lvl;
};

/*
 * GLOBAL VARIABLES DECLARATIONS
 ****************************************************************************************
 */

/// Blood Pressure Application environment
extern struct app_blps_env_tag app_blps_env;

/// Table of message handlers
extern const struct ke_state_handler app_blps_table_handler;

/*
 * FUNCTIONS DECLARATION
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 *
 * Blood Pressure Application Functions
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Initialize Blood Pressure Application Module
 ****************************************************************************************
 */
void app_blps_init(void);

/**
 ****************************************************************************************
 * @brief Add a Blood Pressure Service instance in the DB
 ****************************************************************************************
 */
void app_blps_add_bp(void);

void app_blps_send_data(void);

#if(BQB_TEST_ENABLE == 1)
  void app_blps_set_feature_val(uint16_t val);
#endif

/// @} APP

#endif // APP_BATT_H_
