/**
 ****************************************************************************************
 *
 * @file app_glps.c
 *
 * @brief Glucose Profile Module entry point
 *
 * Copyright (C) Beken 2009-2016
 *
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup APP
 * @{
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration

#if (BLE_GL_SENSOR)

/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include <string.h>
#include "app_glps.h"                // 
#include "app.h"                     // Application Definitions
#include "app_task.h"                // application task definitions
#include "glps_task.h"               // health thermometer functions
#include "co_bt.h"
#include "prf_types.h"               // Profile common types definition
#include "arch.h"                    // Platform Definitions
#include "prf.h"
#include "glps.h"
#include "ke_timer.h"
#include "uart.h"

#include "user_config.h"

/*
 * DEFINES
 ****************************************************************************************
 */

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Glucose Profile Application Module Environment Structure
struct app_glps_env_tag app_glps_env;

/*
 * GLOBAL FUNCTION DEFINITIONS
 ****************************************************************************************
 */

void app_glps_init(void)
{
    // Reset the environment
    memset(&app_glps_env, 0, sizeof(struct app_glps_env_tag));

    // Initial ....
    DBG_PRINTF("[GLPS] %s \r\n", __func__);
}

void app_glps_add_gl(void)
{
    struct glps_db_cfg* db_cfg;
    // Allocate
    struct gapm_profile_task_add_cmd *req = KE_MSG_ALLOC_DYN(GAPM_PROFILE_TASK_ADD_CMD,
                                                  TASK_GAPM, TASK_APP,
                                                  gapm_profile_task_add_cmd, sizeof(struct glps_db_cfg));
    DBG_PRINTF("[GLPS] %s \r\n", __func__);
    // Fill message
    req->operation   = GAPM_PROFILE_TASK_ADD;
    req->sec_lvl     = 0;
    req->prf_task_id = TASK_ID_GLPS;
    req->app_task    = TASK_APP;
    req->start_hdl   = 0;

    // Set parameters
    db_cfg = (struct glps_db_cfg* ) req->param;
 
    // 
    db_cfg->features = 1;
    db_cfg->meas_ctx_supported = 0;
    // Send the message
    ke_msg_send(req);
#if(BQB_TEST_ENABLE == 1)
	void app_glps_test_db_init(uint8_t);
	app_glps_test_db_init(3);
#endif
}


static int glps_enable_rsp_handler(ke_msg_id_t const msgid,
                                    struct glps_enable_rsp const *param,
                                    ke_task_id_t const dest_id,
                                    ke_task_id_t const src_id)
{
    DBG_PRINTF("[GLPS] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

/**
 ****************************************************************************************
 * @brief
 *
 * @param[in] msgid     Id of the message received.
 * @param[in] param     Pointer to the parameters of the message.
 * @param[in] dest_id   ID of the receiving task instance (TASK_GAP).
 * @param[in] src_id    ID of the sending task instance.
 *
 * @return If the message was consumed or not.
 ****************************************************************************************
 */
static int app_glps_msg_dflt_handler(ke_msg_id_t const msgid,
                                     void const *param,
                                     ke_task_id_t const dest_id,
                                     ke_task_id_t const src_id)
{
    // Drop the message
    DBG_PRINTF("[GLPS] %s \r\n", __func__);
    
    return (KE_MSG_CONSUMED);
}

#if(BQB_TEST_ENABLE == 1)	////20210125
	////Create test data base
	struct app_glps_test_db_tag glps_test_db;
	struct app_glps_period_operation_tag glps_test_op;
	void app_glps_test_db_init(uint8_t data_size)
	{
		uint8_t tmpCnt = 0;
		memset(&glps_test_db, 0, sizeof(struct app_glps_test_db_tag));
		memset(&glps_test_op, 0, sizeof(struct app_glps_period_operation_tag));
		data_size &= _APP_GLPS_TEST_DB_SIZE_MAX;

		for(tmpCnt = 0; tmpCnt<data_size; tmpCnt++)
		{
			glps_test_db.meas_tab[tmpCnt].meas.base_time.year = 2021;
			glps_test_db.meas_tab[tmpCnt].meas.base_time.month = 1+tmpCnt;	////1;
			glps_test_db.meas_tab[tmpCnt].meas.base_time.day = 1;
			glps_test_db.meas_tab[tmpCnt].meas.base_time.hour = 12;
			glps_test_db.meas_tab[tmpCnt].meas.base_time.min = 0;
			glps_test_db.meas_tab[tmpCnt].meas.base_time.sec = 0;
			glps_test_db.meas_tab[tmpCnt].meas.time_offset = 1;		////50*tmpCnt;
			glps_test_db.meas_tab[tmpCnt].meas.flags = 0x01;		////Time Offset present
			glps_test_db.meas_tab[tmpCnt].seq_num = (tmpCnt+1);
			glps_test_db.data_valid |= (1<<tmpCnt);
		}
		glps_test_db.racp_ctrl.data_cnt = tmpCnt;
		glps_test_db.racp_ctrl.last_seq_num = tmpCnt;
	}

	////Add for Notify glps data or Indicate
	void app_glps_send_data(uint8_t evt_cfg, uint8_t *param)
	{
		if(evt_cfg & GLPS_IND_NTF_EVT(GLS_IDX_MEAS_NTF_CFG))
		{	////param[0]: Data Index
			struct glps_send_meas_with_ctx_cmd *req = KE_MSG_ALLOC(GLPS_SEND_MEAS_WITHOUT_CTX_CMD,
													  prf_get_task_from_id(TASK_ID_GLPS),
													  TASK_APP,		////Or TASK_ID_GLPS
													  glps_send_meas_with_ctx_cmd);
			memcpy(req, &glps_test_db.meas_tab[(param[0]&0x03)], sizeof(struct glps_send_meas_with_ctx_cmd));
			// Send the message
			ke_msg_send(req);
		}
		if(evt_cfg & GLPS_IND_NTF_EVT(GLS_IDX_REC_ACCESS_CTRL_IND_CFG))
		{
			struct glps_send_racp_rsp_cmd *req = KE_MSG_ALLOC(GLPS_SEND_RACP_RSP_CMD,
													  prf_get_task_from_id(TASK_ID_GLPS),
													  TASK_APP,		////Or TASK_ID_GLPS
													  glps_send_racp_rsp_cmd);
			if(param[0]>=100)
			{
				req->num_of_record = glps_test_db.racp_ctrl.data_cnt;
				req->op_code = glps_test_db.racp_ctrl.op_code;
				req->status = GLP_RSP_SUCCESS;
			}
			else
			{
				req->num_of_record = param[0];
				req->op_code = param[1];
				req->status = param[2];
			}
			// Send the message
			ke_msg_send(req);
		}
	}


	static void app_glps_send_racp_rsp(uint8_t num_of_rec, uint8_t op_code, uint8_t status)
	{
		struct glps_send_racp_rsp_cmd *req = KE_MSG_ALLOC(GLPS_SEND_RACP_RSP_CMD,
													  prf_get_task_from_id(TASK_ID_GLPS),
													  TASK_APP,		////Or TASK_ID_GLPS		////????
													  glps_send_racp_rsp_cmd);
		req->num_of_record = num_of_rec;
		req->op_code = op_code;
		req->status = status;
	#if 1	////Convert Err code
		if(num_of_rec == 0)
		{
			req->status = GLP_RSP_NO_RECS_FOUND;
		}
	#endif
		ke_msg_send(req);
	}

	static void app_glps_send_meas_rsp(uint8_t data_ind)
	{
		struct glps_send_meas_with_ctx_cmd *req = KE_MSG_ALLOC(GLPS_SEND_MEAS_WITHOUT_CTX_CMD,
													  prf_get_task_from_id(TASK_ID_GLPS),
													  TASK_APP,		////Or TASK_ID_GLPS
													  glps_send_meas_with_ctx_cmd);
		memcpy(req, &glps_test_db.meas_tab[data_ind], sizeof(struct glps_send_meas_with_ctx_cmd));
		// Send the message
		ke_msg_send(req);
	}

	static int8_t app_glps_time_compare(struct prf_date_time *time1, struct prf_date_time *time2)
	{	//// -1, time1 < time2
		////  0, time1 = time2
		////  1, time1 > time2
		uint32_t sec1 = 0;
		uint32_t sec2 = 0;
		int8_t result = 0;

		if(time1->year < time2->year)
		{
			result = -1;
		}
		else if(time1->year > time2->year)
		{
			result = 1;
		}
		else////same year
		{
			if(time1->month < time2->month)
			{
				result = -1;
			}
			else if(time1->month > time2->month)
			{
				result = 1;
			}
			else////same month
			{
				if(time1->day < time2->day)
				{
					result = -1;
				}
				else if(time1->day > time2->day)
				{
					result = 1;
				}
				else////same day
				{
					sec1 = ((time1->hour*60)+time1->min)*60 + time1->sec;
					sec2 = ((time2->hour*60)+time2->min)*60 + time2->sec;
					if(sec1 < sec2)
					{
						result = -1;
					}
					else if(sec1 > sec2)
					{
						result = 1;
					}
					else////same time
					{
						result = 0;
					}
				}
			}
		}
		return result;
	}

	static uint8_t app_glps_get_rsp_meas_data_index(struct glps_racp_req_rcv_ind *param, uint8_t *send_tar)
	{
		uint8_t _operator = param->racp_req.filter.operator;
		uint8_t filter_type = param->racp_req.filter.filter_type;
		uint16_t val = 0;
		uint8_t db_max_size = _APP_GLPS_TEST_DB_SIZE_MAX;
		int8_t tmpCnt = 0;
		uint8_t _status = GLP_RSP_SUCCESS;
		uint8_t s_tar = 0;
		struct prf_date_time *target;
		struct prf_date_time *ref;

		if(_operator == GLP_OP_ALL_RECS)
		{
			s_tar = glps_test_db.data_valid;
		}
		else if(_operator == GLP_OP_LT_OR_EQ)				////smaller than max ???
		{
			if(filter_type == GLP_FILTER_SEQ_NUMBER)
			{
				val = param->racp_req.filter.val.seq_num.max;

				for(tmpCnt = 0; tmpCnt<db_max_size; tmpCnt++)
				{
					if(glps_test_db.data_valid & (1<<tmpCnt))
					{
						if(glps_test_db.meas_tab[tmpCnt].seq_num <= val)
						{
							s_tar |= (1<<tmpCnt);
						}
					}
				}
			}
			else
			{
				target = &param->racp_req.filter.val.time.facetime_max;
				int8_t rslt = 0;
				for(tmpCnt = 0; tmpCnt<db_max_size; tmpCnt++)
				{
					if(glps_test_db.data_valid & (1<<tmpCnt))
					{
						ref = &glps_test_db.meas_tab[tmpCnt].meas.base_time;
						rslt = app_glps_time_compare(ref, target);
						if((rslt == 0) || (rslt == -1))
						{
							s_tar |= (1<<tmpCnt);
						}
					}
				}
			}
		}
		else if(_operator == GLP_OP_GT_OR_EQ)		////bigger than min	???
		{
			if(filter_type == GLP_FILTER_SEQ_NUMBER)
			{
				val = param->racp_req.filter.val.seq_num.min;

				for(tmpCnt = 0; tmpCnt<db_max_size; tmpCnt++)
				{
					if(glps_test_db.data_valid & (1<<tmpCnt))
					{
						if(glps_test_db.meas_tab[tmpCnt].seq_num >= val)
						{
							s_tar |= (1<<tmpCnt);
						}
					}
				}
			}
			else
			{
				target = &param->racp_req.filter.val.time.facetime_min;
				int8_t rslt = 0;
				for(tmpCnt = 0; tmpCnt<db_max_size; tmpCnt++)
				{
					if(glps_test_db.data_valid & (1<<tmpCnt))
					{
						ref = &glps_test_db.meas_tab[tmpCnt].meas.base_time;
						rslt = app_glps_time_compare(ref, target);
						if((rslt == 0) || (rslt == 1))
						{
							s_tar |= (1<<tmpCnt);
						}
					}
				}
			}
		}
		else if(_operator == GLP_OP_WITHIN_RANGE_OF)
		{
			if(filter_type == GLP_FILTER_SEQ_NUMBER)	////Do Not Know what is the range????
			{
				uint16_t val1 = param->racp_req.filter.val.seq_num.min;
				uint16_t val2 = param->racp_req.filter.val.seq_num.max;

				for(tmpCnt = 0; tmpCnt<db_max_size; tmpCnt++)
				{
					if(glps_test_db.data_valid & (1<<tmpCnt))
					{
						if((glps_test_db.meas_tab[tmpCnt].seq_num >= val1) || (glps_test_db.meas_tab[tmpCnt].seq_num <= val2))
						{
							s_tar |= (1<<tmpCnt);
						}
					}
				}
			}
			else
			{
				int8_t rslt = 0;
				for(tmpCnt = 0; tmpCnt<db_max_size; tmpCnt++)
				{
					target = &param->racp_req.filter.val.time.facetime_max;
					if(glps_test_db.data_valid & (1<<tmpCnt))
					{
						ref = &glps_test_db.meas_tab[tmpCnt].meas.base_time;
						rslt = app_glps_time_compare(ref, target);
						if((rslt == 0) || (rslt == -1))
						{
							target = &param->racp_req.filter.val.time.facetime_min;
							rslt = app_glps_time_compare(ref, target);
							if((rslt == 0) || (rslt == 1))
							{
								s_tar |= (1<<tmpCnt);
							}
						}
					}
				}
			}
		}
		else if(_operator == GLP_OP_FIRST_REC)
		{
			////assume sec_num or time has been sorted
			for(tmpCnt = 0; tmpCnt<db_max_size; tmpCnt++)
			{
				if(glps_test_db.data_valid & (1<<tmpCnt))
				{
					s_tar |= (1<<tmpCnt);
					break;
				}
			}
		}
		else if(_operator == GLP_OP_LAST_REC)
		{
			////assume sec_num or time has been sorted
			for(tmpCnt = (db_max_size-1); tmpCnt>=0; tmpCnt--)
			{
				if(glps_test_db.data_valid & (1<<tmpCnt))
				{
					s_tar |= (1<<tmpCnt);
					break;
				}
			}
		}
		else
		{
			_status = GLP_RSP_INVALID_OPERATOR;
		}
		*send_tar = s_tar;

		return _status;
	}

	static uint8_t app_glps_get_bit_count(uint8_t val)
	{
		uint8_t rtnval = 0;
		uint8_t tmpcnt = 0;
		for(tmpcnt=0; tmpcnt<8; tmpcnt++)
		{
			if(val & (1<<tmpcnt))
				rtnval++;
		}
		return rtnval;
	}

	static void app_glps_period_event_handler(void)
	{
		for(; glps_test_op.cur_ind<_APP_GLPS_TEST_DB_SIZE_MAX; glps_test_op.cur_ind++)
		{
			if(glps_test_op.act_ind & (1<<glps_test_op.cur_ind))
			{
				app_glps_send_meas_rsp(glps_test_op.cur_ind);
				glps_test_op.cur_ind++;
				break;
			}
		}
		if(glps_test_op.cur_ind == _APP_GLPS_TEST_DB_SIZE_MAX)
		{
			app_glps_send_racp_rsp(app_glps_get_bit_count(glps_test_op.act_ind), glps_test_op.op_code, GLP_RSP_SUCCESS);
			glps_test_op.act_ind = 0;
			glps_test_op.cur_ind = 0xFF;
			glps_test_op.op_code = 0;
		}
		else
		{
			////ke_timer_set(GLPS_PERIODIC_EVENT_HANDLE, prf_get_task_from_id(TASK_ID_GLPS), 10);
			ke_timer_set(GLPS_PERIODIC_EVENT_HANDLE, TASK_APP, 100);
		}
	}


#endif ////(BQB_TEST_ENABLE == 1)


////Add for Indication and Notifying config action
static int app_glps_cfg_indntf_handler(ke_msg_id_t const msgid,
                                     void const *param,
                                     ke_task_id_t const dest_id,
                                     ke_task_id_t const src_id)
{
	DBG_PRINTF("%s \r\n", __func__);

    return (KE_MSG_CONSUMED);
}

static int app_glps_racp_req_handler(ke_msg_id_t const msgid,
                                     struct glps_racp_req_rcv_ind *param,
                                     ke_task_id_t const dest_id,
                                     ke_task_id_t const src_id)
{
	DBG_PRINTF("%s \r\n", __func__);

#if(BQB_TEST_ENABLE == 1)
	uint8_t op_code = param->racp_req.op_code;
	uint8_t activ_ind = 0;
	uint8_t tmpCnt = 0;
	uint8_t db_max_size = _APP_GLPS_TEST_DB_SIZE_MAX;
	uint8_t _status = GLP_RSP_SUCCESS;
	uint8_t *valid_tar = &glps_test_db.data_valid;
	app_glps_get_rsp_meas_data_index(param, &activ_ind);
	switch(op_code)
	{
		case GLP_REQ_REP_STRD_RECS:
			glps_test_op.op_code = GLP_REQ_REP_STRD_RECS;
			glps_test_op.act_ind = activ_ind;
			glps_test_op.cur_ind = 0;
			ke_timer_set(GLPS_PERIODIC_EVENT_HANDLE, TASK_APP, 50);
			break;

		case GLP_REQ_DEL_STRD_RECS:
			for(tmpCnt=0; tmpCnt<db_max_size; tmpCnt++)
			{
				if((activ_ind & (1<<tmpCnt)) && (*valid_tar & (1<<tmpCnt)))
				{
					*valid_tar &= (~(1<<tmpCnt));
					memset(&glps_test_db.meas_tab[tmpCnt], 0, sizeof(struct glps_send_meas_with_ctx_cmd));
					if(glps_test_db.racp_ctrl.data_cnt > 0)
					{
						glps_test_db.racp_ctrl.data_cnt--;
					}
				}
			}
			app_glps_send_racp_rsp(app_glps_get_bit_count(activ_ind), op_code, _status);
			break;

		case GLP_REQ_ABORT_OP:
			ke_timer_clear(GLPS_PERIODIC_EVENT_HANDLE, TASK_APP);
			app_glps_send_racp_rsp(glps_test_op.act_ind, op_code, GLP_RSP_SUCCESS);
			glps_test_op.act_ind = 0;
			glps_test_op.cur_ind = 0xFF;
			glps_test_op.op_code = 0;
			break;

		case GLP_REQ_REP_NUM_OF_STRD_RECS:	////Filter
			app_glps_send_racp_rsp(app_glps_get_bit_count(activ_ind), op_code, _status);
			break;

		case GLP_REQ_NUM_OF_STRD_RECS_RSP:
			break;

		default:
			app_glps_send_racp_rsp(0, op_code, GLP_RSP_OP_CODE_NOT_SUP);
			break;
	}

#endif	////(BQB_TEST_ENABLE == 1)
	return (KE_MSG_CONSUMED);
}

#if(BQB_TEST_ENABLE == 1)
  #include "prf_utils.h"
  void app_glps_set_feature_val(uint16_t val)
  {
	struct glps_env_tag *glps_env = PRF_ENV_GET(GLPS, glps);
    glps_env->features = val;
  }

  void app_glps_add_data(void)
  {

  }
#endif

/*
 * LOCAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Default State handlers definition
const struct ke_msg_handler app_glps_msg_handler_list[] =
{
    // Note: first message is latest message checked by kernel so default is put on top.
    {KE_MSG_DEFAULT_HANDLER,        (ke_msg_func_t)app_glps_msg_dflt_handler},
	{GLPS_ENABLE_RSP,       		(ke_msg_func_t)glps_enable_rsp_handler},
	{GLPS_CFG_INDNTF_IND,       	(ke_msg_func_t)app_glps_cfg_indntf_handler},
    {GLPS_RACP_REQ_RCV_IND, 		(ke_msg_func_t)app_glps_racp_req_handler},
#if(BQB_TEST_ENABLE == 1)
	{GLPS_PERIODIC_EVENT_HANDLE,	(ke_msg_func_t)app_glps_period_event_handler}
#endif
};

const struct ke_state_handler app_glps_table_handler =
    {&app_glps_msg_handler_list[0], (sizeof(app_glps_msg_handler_list)/sizeof(struct ke_msg_handler))};

    
#endif //#if (BLE_GL_SENSOR)    

/// @} APP
