/**
 ****************************************************************************************
 *
 * @file app_glps.h
 *
 * @brief Glucose Module entry point
 *
 * Copyright (C) 
 *
 *
 ****************************************************************************************
 */

#ifndef APP_GLPS_H_
#define APP_GLPS_H_

/**
 ****************************************************************************************
 * @addtogroup APP
 * @ingroup RICOW
 *
 * @brief Glucose Application Module entry point
 *
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration


#include <stdint.h>          // Standard Integer Definition
#include "ke_task.h"         // Kernel Task Definition

#include "user_config.h"

#define APP_GLP_ADV_DATA_UUID        "\x03\x03\x18\x2A"
#define APP_GLP_ADV_DATA_UUID_LEN    (4)


/*
 * STRUCTURES DEFINITION
 ****************************************************************************************
 */

/// Glucose Application Module Environment Structure
struct app_glps_env_tag
{
    /// Connection handle
    uint8_t conidx;
    /// Current Battery Level
    uint8_t glps_lvl;
};


#if(BQB_TEST_ENABLE == 1)
	#include "glps_task.h"
	#define  _APP_GLPS_TEST_DB_SIZE_MAX	3
	struct app_glps_racp_ctrl_tag
	{
		uint8_t data_cnt;
		int8_t last_seq_num;
		uint8_t op_code;
	};

	struct app_glps_test_db_tag
	{
		struct app_glps_racp_ctrl_tag racp_ctrl;
		struct glps_send_meas_with_ctx_cmd meas_tab[_APP_GLPS_TEST_DB_SIZE_MAX];
		uint8_t data_valid;
	};

	struct app_glps_period_operation_tag
	{
		uint8_t op_code;
		uint8_t act_ind;
		uint8_t cur_ind;
	};
#endif

/*
 * GLOBAL VARIABLES DECLARATIONS
 ****************************************************************************************
 */

/// Glucose Application environment
extern struct app_glps_env_tag app_glps_env;

/// Table of message handlers
extern const struct ke_state_handler app_glps_table_handler;

/*
 * FUNCTIONS DECLARATION
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 *
 * Glucose Application Functions
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Initialize Glucose Application Module
 ****************************************************************************************
 */
void app_glps_init(void);

/**
 ****************************************************************************************
 * @brief Add a Glucose Service instance in the DB
 ****************************************************************************************
 */
void app_glps_add_gl(void);




/// @} APP

#endif // APP_BATT_H_
