/**
 ****************************************************************************************
 *
 * @file app_hrps.c
 *
 * @brief Heart Rate Profile Module entry point
 *
 * Copyright (C) Beken 2009-2016
 *
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup APP
 * @{
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration

#if (BLE_HR_SENSOR)

/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include <string.h>
#include "app_hrps.h"                // 
#include "app.h"                     // Application Definitions
#include "app_task.h"                // application task definitions
#include "hrps_task.h"               // health thermometer functions
#include "co_bt.h"
#include "prf_types.h"               // Profile common types definition
#include "arch.h"                    // Platform Definitions
#include "prf.h"
#include "hrps.h"

#include "uart.h"

/*
 * DEFINES
 ****************************************************************************************
 */

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Heart Rate Profile Application Module Environment Structure
struct app_hrps_env_tag app_hrps_env;

/*
 * GLOBAL FUNCTION DEFINITIONS
 ****************************************************************************************
 */

void app_hrps_init(void)
{
    // Reset the environment
    memset(&app_hrps_env, 0, sizeof(struct app_hrps_env_tag));

    // Initial ....
    DBG_PRINTF("[HRPS] %s \r\n", __func__);
}

void app_hrps_add_hr(void)
{
    struct hrps_db_cfg* db_cfg;
    // Allocate 
    struct gapm_profile_task_add_cmd *req = KE_MSG_ALLOC_DYN(GAPM_PROFILE_TASK_ADD_CMD,
                                                  TASK_GAPM, TASK_APP,
                                                  gapm_profile_task_add_cmd, sizeof(struct hrps_db_cfg));
    DBG_PRINTF("[HRPS] %s \r\n", __func__);
    // Fill message
    req->operation   = GAPM_PROFILE_TASK_ADD;
    req->sec_lvl     = 0;
    req->prf_task_id = TASK_ID_HRPS;
    req->app_task    = TASK_APP;
    req->start_hdl   = 0;

    // Set parameters
    db_cfg = (struct hrps_db_cfg* ) req->param;
    
    db_cfg->features = HRPS_BODY_SENSOR_LOC_CHAR_SUP | HRPS_HR_MEAS_NTF_CFG ;
    db_cfg->body_sensor_loc = 95;
 

    // Send the message
    ke_msg_send(req);
}


static int heart_rate_meas_send_rsp_handler(ke_msg_id_t const msgid,
                                    struct hrps_meas_send_req const *param,
                                    ke_task_id_t const dest_id,
                                    ke_task_id_t const src_id)
{
    DBG_PRINTF("[HRPS]%s (%x,%x)\r\n", __func__,dest_id,src_id);

    return (KE_MSG_CONSUMED);
}
                                        


//Heart Rate Measurement
static int heart_rate_meas_ntf_cfg_ind_handler(ke_msg_id_t const msgid,
                                               struct hrps_cfg_indntf_ind const *param,
                                               ke_task_id_t const dest_id,
                                               ke_task_id_t const src_id)
{
    DBG_PRINTF("[HRPS]%s (%x,%x)\r\n", __func__,param->conidx,param->cfg_val);
    
#if BQB_TEST_ENABLE  
    // Test Case : HRS/SEN/CN/BV-01-C
    if(param->cfg_val == 1)
    {
    
    struct hrps_meas_send_req *req = KE_MSG_ALLOC(  HRPS_MEAS_SEND_REQ,
                                                    prf_get_task_from_id(TASK_ID_HRPS),
                                                    TASK_APP,
                                                    hrps_meas_send_req);    
        req->meas_val.energy_expended = 0;
        req->meas_val.flags = 0;
        req->meas_val.heart_rate = 80;
        req->meas_val.nb_rr_interval = 1;
        req->meas_val.rr_intervals[0] = 10;        
    
        ke_msg_send(req);
    }
#endif    
        
    
    return (KE_MSG_CONSUMED);
}



static int hrps_enable_rsp_handler(ke_msg_id_t const msgid,
                                    struct hrps_enable_rsp const *param,
                                    ke_task_id_t const dest_id,
                                    ke_task_id_t const src_id)
{
    DBG_PRINTF("[HRPS] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

/**
 ****************************************************************************************
 * @brief
 *
 * @param[in] msgid     Id of the message received.
 * @param[in] param     Pointer to the parameters of the message.
 * @param[in] dest_id   ID of the receiving task instance (TASK_GAP).
 * @param[in] src_id    ID of the sending task instance.
 *
 * @return If the message was consumed or not.
 ****************************************************************************************
 */
static int app_hrps_msg_dflt_handler(ke_msg_id_t const msgid,
                                     void const *param,
                                     ke_task_id_t const dest_id,
                                     ke_task_id_t const src_id)
{
    // Drop the message
    DBG_PRINTF("[HRPS] %s \r\n", __func__);
    
    return (KE_MSG_CONSUMED);
}

/*
 * LOCAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Default State handlers definition
const struct ke_msg_handler app_hrps_msg_handler_list[] =
{
    // Note: first message is latest message checked by kernel so default is put on top.
    {KE_MSG_DEFAULT_HANDLER,        (ke_msg_func_t)app_hrps_msg_dflt_handler},
    {HRPS_CFG_INDNTF_IND,           (ke_msg_func_t)heart_rate_meas_ntf_cfg_ind_handler},
    {HRPS_MEAS_SEND_RSP,            (ke_msg_func_t)heart_rate_meas_send_rsp_handler},
	{HRPS_ENABLE_RSP,       		(ke_msg_func_t)hrps_enable_rsp_handler},
		
};

const struct ke_state_handler app_hrps_table_handler =
    {&app_hrps_msg_handler_list[0], (sizeof(app_hrps_msg_handler_list)/sizeof(struct ke_msg_handler))};

    
#endif //#if (BLE_HR_SENSOR)    

/// @} APP
