/**
 ****************************************************************************************
 *
 * @file app_hrps.h
 *
 * @brief Heart Rate Module entry point
 *
 * Copyright (C) 
 *
 *
 ****************************************************************************************
 */

#ifndef APP_HRPS_H_
#define APP_HRPS_H_

/**
 ****************************************************************************************
 * @addtogroup APP
 * @ingroup RICOW
 *
 * @brief Heart Rate Application Module entry point
 *
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration


#include <stdint.h>          // Standard Integer Definition
#include "ke_task.h"         // Kernel Task Definition

/*
 * STRUCTURES DEFINITION
 ****************************************************************************************
 */

/// Heart Rate Application Module Environment Structure
struct app_hrps_env_tag
{
    /// Connection handle
    uint8_t conidx;
    /// Current Battery Level
    uint8_t hrps_lvl;
};

/*
 * GLOBAL VARIABLES DECLARATIONS
 ****************************************************************************************
 */

/// Heart Rate Application environment
extern struct app_hrps_env_tag app_hrps_env;

/// Table of message handlers
extern const struct ke_state_handler app_hrps_table_handler;

/*
 * FUNCTIONS DECLARATION
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 *
 * Heart Rate Application Functions
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Initialize Heart Rate Application Module
 ****************************************************************************************
 */
void app_hrps_init(void);

/**
 ****************************************************************************************
 * @brief Add a Heart Rate Service instance in the DB
 ****************************************************************************************
 */
void app_hrps_add_hr(void);




/// @} APP

#endif // APP_BATT_H_
