/**
 ****************************************************************************************
 *
 * @file app_htpt.c
 *
 * @brief Health Thermometer Module entry point
 *
 * Copyright (C) Beken 2009-2016
 *
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup APP
 * @{
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration

#if (BLE_HT_THERMOM)

/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include <string.h>
#include "app_htpt.h"                // 
#include "app.h"                     // Application Definitions
#include "app_task.h"                // application task definitions
#include "htpt_task.h"               // health thermometer functions
#include "co_bt.h"
#include "prf_types.h"               // Profile common types definition
#include "arch.h"                    // Platform Definitions
#include "prf.h"
#include "htpt.h"

#include "uart.h"

/*
 * DEFINES
 ****************************************************************************************
 */

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Health Thermometer Application Module Environment Structure
struct app_htpt_env_tag app_htpt_env;

/*
 * GLOBAL FUNCTION DEFINITIONS
 ****************************************************************************************
 */

void app_htpt_init(void)
{
    // Reset the environment
    memset(&app_htpt_env, 0, sizeof(struct app_htpt_env_tag));

    // Initial ....
    DBG_PRINTF("[HTPT] %s \r\n", __func__);
}

void app_htpt_add_ht(void)
{
    struct htpt_db_cfg* db_cfg;
    // Allocate the 
    struct gapm_profile_task_add_cmd *req = KE_MSG_ALLOC_DYN(GAPM_PROFILE_TASK_ADD_CMD,
                                                  TASK_GAPM, TASK_APP,
                                                  gapm_profile_task_add_cmd, sizeof(struct htpt_db_cfg));
    DBG_PRINTF("[HTPT] %s \r\n", __func__);
    // Fill message
    req->operation   = GAPM_PROFILE_TASK_ADD;
    req->sec_lvl     = 0;
    req->prf_task_id = TASK_ID_HTPT;
    req->app_task    = TASK_APP;
    req->start_hdl   = 0;

    // Set parameters
    db_cfg = (struct htpt_db_cfg* ) req->param;
 
    // Health thermometer Feature
    db_cfg->features = HTPT_ALL_FEAT_SUP;//HTPT_TEMP_TYPE_CHAR_SUP;
    // Temperature Type Value
    db_cfg->temp_type = 1;
    // 
    db_cfg->valid_range_min = HTPT_MEAS_INTV_DFLT_MIN;
    db_cfg->valid_range_max = HTPT_MEAS_INTV_DFLT_MAX;
    db_cfg->meas_intv = HTPT_MEAS_INTV_DFLT_MAX;
    
    // Send the message
    ke_msg_send(req);
}


static int htpt_enable_rsp_handler(ke_msg_id_t const msgid,
                                    struct htpt_enable_rsp const *param,
                                    ke_task_id_t const dest_id,
                                    ke_task_id_t const src_id)
{
    DBG_PRINTF("[HTPT] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}


static int htpt_temp_send_rsp_handler(ke_msg_id_t const msgid,
                                        struct htpt_temp_send_rsp const *param,
                                        ke_task_id_t const dest_id,
                                        ke_task_id_t const src_id)
{
    DBG_PRINTF("[HTPT] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

static int htpt_meas_intv_upd_handler(ke_msg_id_t const msgid,
                                        struct htpt_meas_intv_upd_rsp const *param,
                                        ke_task_id_t const dest_id,
                                        ke_task_id_t const src_id)
{
    DBG_PRINTF("[HTPT] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

static int htpt_meas_intv_chg_reg_ind_handler(ke_msg_id_t const msgid,
                                                struct htpt_meas_intv_chg_req_ind const *param,
                                                ke_task_id_t const dest_id,
                                                ke_task_id_t const src_id)
{
    DBG_PRINTF("[HTPT] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

static int htpt_cfg_indntf_ind_handler(ke_msg_id_t const msgid,
                                        struct htpt_cfg_indntf_ind const *param,
                                        ke_task_id_t const dest_id,
                                        ke_task_id_t const src_id)
{
    DBG_PRINTF("[HTPT] %s (%x,%x)\r\n", __func__,param->conidx,param->ntf_ind_cfg);
    

#if BQB_TEST_ENABLE  
    // Test Case : HRS/SEN/CN/BV-01-C
    if(param->ntf_ind_cfg == 1)
    {
    
    struct htpt_temp_send_req *req = KE_MSG_ALLOC(  HTPT_TEMP_SEND_REQ,
                                                    prf_get_task_from_id(TASK_ID_HTPT),
                                                    TASK_APP,
                                                    htpt_temp_send_req);    
        req->temp_meas.temp = 36;
        req->temp_meas.flags = HTP_FLAG_CELSIUS;
        req->temp_meas.type =  HTP_TYPE_BODY;
        req->temp_meas.time_stamp.year = 2021;
        req->temp_meas.time_stamp.month = 1;
        req->temp_meas.time_stamp.day = 1;
        req->temp_meas.time_stamp.hour = 12;
        req->temp_meas.time_stamp.min = 0;
        req->temp_meas.time_stamp.sec = 0;   

        req->stable_meas = 1;    
        ke_msg_send(req);
    }
#endif  

    
    return (KE_MSG_CONSUMED);
}




/**
 ****************************************************************************************
 * @brief
 *
 * @param[in] msgid     Id of the message received.
 * @param[in] param     Pointer to the parameters of the message.
 * @param[in] dest_id   ID of the receiving task instance (TASK_GAP).
 * @param[in] src_id    ID of the sending task instance.
 *
 * @return If the message was consumed or not.
 ****************************************************************************************
 */
static int app_htpt_msg_dflt_handler(ke_msg_id_t const msgid,
                                     void const *param,
                                     ke_task_id_t const dest_id,
                                     ke_task_id_t const src_id)
{
    // Drop the message
    DBG_PRINTF("[HTPT] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

/*
 * LOCAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Default State handlers definition
const struct ke_msg_handler app_htpt_msg_handler_list[] =
{
    // Note: first message is latest message checked by kernel so default is put on top.
    {KE_MSG_DEFAULT_HANDLER,        (ke_msg_func_t)app_htpt_msg_dflt_handler},
	{HTPT_ENABLE_RSP,       		(ke_msg_func_t)htpt_enable_rsp_handler},
	{HTPT_TEMP_SEND_RSP,       		(ke_msg_func_t)htpt_temp_send_rsp_handler},
 	{HTPT_MEAS_INTV_UPD_RSP,        (ke_msg_func_t)htpt_meas_intv_upd_handler},   
 	{HTPT_MEAS_INTV_CHG_REQ_IND,    (ke_msg_func_t)htpt_meas_intv_chg_reg_ind_handler},   
 	{HTPT_CFG_INDNTF_IND,           (ke_msg_func_t)htpt_cfg_indntf_ind_handler},   
};

const struct ke_state_handler app_htpt_table_handler =
    {&app_htpt_msg_handler_list[0], (sizeof(app_htpt_msg_handler_list)/sizeof(struct ke_msg_handler))};

    
#endif //#if (BLE_HT_THERMOM)    

/// @} APP
