/**
 ****************************************************************************************
 *
 * @file app_htpt.h
 *
 * @brief Health Thermometer Module entry point
 *
 * Copyright (C) 
 *
 *
 ****************************************************************************************
 */

#ifndef APP_HTPT_H_
#define APP_HTPT_H_

/**
 ****************************************************************************************
 * @addtogroup APP
 * @ingroup RICOW
 *
 * @brief Health Thermometer Application Module entry point
 *
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration


#include <stdint.h>          // Standard Integer Definition
#include "ke_task.h"         // Kernel Task Definition

/*
 * STRUCTURES DEFINITION
 ****************************************************************************************
 */

/// Health Thermometer Application Module Environment Structure
struct app_htpt_env_tag
{
    /// Connection handle
    uint8_t conidx;
    /// Current Battery Level
    uint8_t htpt_lvl;
};

/*
 * GLOBAL VARIABLES DECLARATIONS
 ****************************************************************************************
 */

/// Health Thermometer Application environment
extern struct app_htpt_env_tag app_htpt_env;

/// Table of message handlers
extern const struct ke_state_handler app_htpt_table_handler;

/*
 * FUNCTIONS DECLARATION
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 *
 * Health Thermometer Application Functions
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Initialize Health Thermometer Application Module
 ****************************************************************************************
 */
void app_htpt_init(void);

/**
 ****************************************************************************************
 * @brief Add a Health Thermometer Service instance in the DB
 ****************************************************************************************
 */
void app_htpt_add_ht(void);




/// @} APP

#endif // APP_BATT_H_
