/**
 ****************************************************************************************
 *
 * @file app_wscs.c
 *
 * @brief Weight SCale Module entry point
 *
 * Copyright (C) Beken 2009-2016
 *
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup APP
 * @{
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration

#if (BLE_WSC_SERVER)

/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include <string.h>
#include "app_wscs.h"                // 
#include "app.h"                     // Application Definitions
#include "app_task.h"                // application task definitions
#include "wscs_task.h"               // health thermometer functions
#include "co_bt.h"
#include "prf_types.h"               // Profile common types definition
#include "arch.h"                    // Platform Definitions
#include "prf.h"
#include "wscs.h"

#include "uart.h"
#include "app_sec.h"
/*
 * DEFINES
 ****************************************************************************************
 */

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Weight SCale Application Module Environment Structure
struct app_wscs_env_tag app_wscs_env;

/// app wscs param environment
struct app_wscs_param app_wscs_param_evn;

/*
 * GLOBAL FUNCTION DEFINITIONS
 ****************************************************************************************
 */

void app_wscs_init(void)
{
    // Reset the environment
    memset(&app_wscs_env, 0, sizeof(struct app_wscs_env_tag));

    // Initial ....
    DBG_PRINTF("[WSCS] %s \r\n", __func__);
}

void app_wscs_add_ws(void)
{
    struct wscs_db_cfg* db_cfg;
    // Allocate 
    struct gapm_profile_task_add_cmd *req = KE_MSG_ALLOC_DYN(GAPM_PROFILE_TASK_ADD_CMD,
                                                  TASK_GAPM, TASK_APP,
                                                  gapm_profile_task_add_cmd, sizeof(struct wscs_db_cfg));
    DBG_PRINTF("[WSCS] %s \r\n", __func__);
    req->operation   = GAPM_PROFILE_TASK_ADD;
    req->sec_lvl     = 0;
    req->prf_task_id = TASK_ID_WSCS;
    req->app_task    = TASK_APP;
    req->start_hdl   = 0;

    // Set parameters
    db_cfg = (struct wscs_db_cfg* ) req->param; 
    
    db_cfg->feature =   WSC_FEAT_TIME_STAMP_SUPP_BIT | WSC_FEAT_BMI_SUPP_BIT //| WSC_FEAT_MULTIPLE_USERS_SUPP_BIT
                        //| (WSC_WGHT_RESOL_05kg_1lb     << WSC_FEAT_WGHT_RESOL_LSB)
                        //| (WSC_HGHT_RESOL_001mtr_1inch << WSC_FEAT_HGHT_RESOL_LSB)
                        ;                       
    
    db_cfg->bcs_start_hdl = 0;

    // Send the message
    ke_msg_send(req);
}

void app_wscs_meas_send(void)
{
    struct wscs_meas_indicate_cmd *ind = KE_MSG_ALLOC(  WSCS_MEAS_INDICATE_CMD,
                                                prf_get_task_from_id(TASK_ID_WSCS),
                                                TASK_APP,
                                                wscs_meas_indicate_cmd);    
    ind->flags = 0 //WSC_MEAS_FLAGS_UNITS_IMPERIAL_BIT 
                    | WSC_MEAS_FLAGS_TIMESTAMP_PRESENT_BIT 
                    | WSC_MEAS_FLAGS_USER_ID_PRESENT_BIT 
                    | WSC_MEAS_FLAGS_BMI_PRESENT_BIT
                ;
    ind->weight = 75;       
    ind->time_stamp.year = 2021; ind->time_stamp.month = 1; ind->time_stamp.day = 1;
    ind->time_stamp.hour = 12; ind->time_stamp.min = 0; ind->time_stamp.sec = 0;
    ind->user_id = 0;
    ind->bmi = 10;
    ind->height = 172;
    ke_msg_send(ind);
}    


static int wscs_meas_ccc_ind_handler( ke_msg_id_t const msgid,
                                        struct wscs_wr_ccc_ind const *param,
                                        ke_task_id_t const dest_id,
                                        ke_task_id_t const src_id)
{
    DBG_PRINTF("[WSCS] %s (%x)\r\n", __func__,(param->ind_cfg));
    
    app_wscs_param_evn.att_ccc_val = param->ind_cfg; 
#if BQB_TEST_ENABLE     
    if(param->ind_cfg == ATT_CCC_START_IND)
    {
		if(app_sec_get_bond_status())	////PTS WSP/SEN/WST/BI-01-I
		{	
        app_wscs_meas_send();
		}
    }       
#endif    
    return (KE_MSG_CONSUMED);
}


static int wscs_cmp_evt_handler( ke_msg_id_t const msgid,
                                    struct wscs_cmp_evt const *param,
                                    ke_task_id_t const dest_id,
                                    ke_task_id_t const src_id)
{
    DBG_PRINTF("[WSCS] %s (%x,%x)\r\n", __func__,param->status,param->operation);

#if 0//BQB_TEST_ENABLE     
    if(param->operation == 1)
    {
        app_wscs_meas_send();
    }       
#endif  
    
    
    return (KE_MSG_CONSUMED);
}



static int wscs_enable_rsp_handler(ke_msg_id_t const msgid,
                                    struct wscs_enable_rsp const *param,
                                    ke_task_id_t const dest_id,
                                    ke_task_id_t const src_id)
{
    DBG_PRINTF("[WSCS] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

/**
 ****************************************************************************************
 * @brief
 *
 * @param[in] msgid     Id of the message received.
 * @param[in] param     Pointer to the parameters of the message.
 * @param[in] dest_id   ID of the receiving task instance (TASK_GAP).
 * @param[in] src_id    ID of the sending task instance.
 *
 * @return If the message was consumed or not.
 ****************************************************************************************
 */
static int app_wscs_msg_dflt_handler(ke_msg_id_t const msgid,
                                     void const *param,
                                     ke_task_id_t const dest_id,
                                     ke_task_id_t const src_id)
{
    // Drop the message
    DBG_PRINTF("[WSCS] %s \r\n", __func__);
    return (KE_MSG_CONSUMED);
}

/*
 * LOCAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Default State handlers definition
const struct ke_msg_handler app_wscs_msg_handler_list[] =
{
    // Note: first message is latest message checked by kernel so default is put on top.
    {KE_MSG_DEFAULT_HANDLER,        (ke_msg_func_t)app_wscs_msg_dflt_handler},
	{WSCS_ENABLE_RSP,       		(ke_msg_func_t)wscs_enable_rsp_handler},    
    {WSCS_MEAS_CCC_IND,       		(ke_msg_func_t)wscs_meas_ccc_ind_handler},    
    {WSCS_CMP_EVT,       		    (ke_msg_func_t)wscs_cmp_evt_handler},    
	
};

const struct ke_state_handler app_wscs_table_handler =
    {&app_wscs_msg_handler_list[0], (sizeof(app_wscs_msg_handler_list)/sizeof(struct ke_msg_handler))};

    
#endif //#if (BLE_WSC_SERVER)    

/// @} APP
