/**
 ****************************************************************************************
 *
 * @file app_wscs.h
 *
 * @brief Weight SCale Module entry point
 *
 * Copyright (C) 
 *
 *
 ****************************************************************************************
 */

#ifndef APP_WSCS_H_
#define APP_WSCS_H_

/**
 ****************************************************************************************
 * @addtogroup APP
 * @ingroup RICOW
 *
 * @brief Weight SCale Application Module entry point
 *
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW configuration


#include <stdint.h>          // Standard Integer Definition
#include "ke_task.h"         // Kernel Task Definition

/*
 * STRUCTURES DEFINITION
 ****************************************************************************************
 */

/// Weight SCale Application Module Environment Structure
struct app_wscs_env_tag
{
    /// Connection handle
    uint8_t conidx;
    /// Current Battery Level
    uint8_t wscs_lvl;
};

/// app wscs param Structure
struct app_wscs_param
{
    /// Char. Client Characteristic Configuration
    uint8_t att_ccc_val;

};


/*
 * GLOBAL VARIABLES DECLARATIONS
 ****************************************************************************************
 */

/// Weight SCale Application environment
extern struct app_wscs_env_tag app_wscs_env;

/// Table of message handlers
extern const struct ke_state_handler app_wscs_table_handler;


/// 
extern struct app_wscs_param app_wscs_used;

/*
 * FUNCTIONS DECLARATION
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 *
 * Weight SCale Application Functions
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Initialize Weight SCale Application Module
 ****************************************************************************************
 */
void app_wscs_init(void);

/**
 ****************************************************************************************
 * @brief Add a Weight SCale Service instance in the DB
 ****************************************************************************************
 */
void app_wscs_add_ws(void);




/// @} APP

#endif // APP_BATT_H_
